"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRebaser = void 0;
const parse5_html_rewriting_stream_1 = __importDefault(require("parse5-html-rewriting-stream"));
const source_map_1 = require("source-map");
const events_1 = require("events");
const url_1 = require("url");
const path_1 = require("path");
const slash_1 = __importDefault(require("slash"));
const stream_1 = require("stream");
const css_source_map_rebase_1 = require("css-source-map-rebase");
const createRebaser = (map, options = {}) => {
    const eventEmitter = new events_1.EventEmitter();
    const rebase = (html) => {
        const rewritingStream = new parse5_html_rewriting_stream_1.default();
        let { rebase } = options;
        let regions = null;
        return new Promise((resolve, reject) => {
            let data = Buffer.from('');
            const outputStream = new stream_1.Writable({
                write(chunk, _encoding, callback) {
                    data = Buffer.concat([data, chunk]);
                    callback();
                }
            });
            outputStream.on('finish', () => {
                resolve({
                    data,
                    map
                });
            });
            rewritingStream.pipe(outputStream);
            const isRebasable = (url) => {
                return !(0, path_1.isAbsolute)(url.href) && (url.host === null) && ((url.hash === null) || (url.path !== null));
            };
            let queue = Promise.resolve();
            const defer = (execution) => {
                queue = queue
                    .then(execution)
                    .catch((error) => {
                    reject(error);
                });
            };
            const getRegions = () => {
                if (!regions) {
                    const foundRegions = [];
                    let sourceMapConsumer = new source_map_1.SourceMapConsumer(JSON.parse(map.toString()));
                    let region = null;
                    let currentSource = null;
                    sourceMapConsumer.eachMapping((mapping) => {
                        let source = mapping.source;
                        if (source !== currentSource) {
                            if (region) {
                                region.endLine = mapping.generatedLine;
                                region.endColumn = mapping.generatedColumn;
                            }
                            region = {
                                source: source,
                                startLine: mapping.generatedLine,
                                startColumn: mapping.generatedColumn,
                                endLine: null,
                                endColumn: null
                            };
                            foundRegions.push(region);
                            currentSource = source;
                        }
                    }, null);
                    regions = foundRegions;
                }
                return regions;
            };
            const findRegion = (startLine, startColumn) => {
                let i = 0;
                let result = null;
                const regions = getRegions();
                const tagStartLine = startLine;
                const tagStartColumn = startColumn - 1;
                while ((i < regions.length) && (result === null)) {
                    let region = regions[i];
                    if (((region.startLine < tagStartLine) || ((region.startLine === tagStartLine) && (region.startColumn <= tagStartColumn))) &&
                        ((region.endLine === null) || (region.endLine > tagStartLine) ||
                            ((region.endLine === tagStartLine) && (region.endColumn === null || (region.endColumn >= tagStartColumn))))) {
                        result = region;
                    }
                    i++;
                }
                return result;
            };
            const transformText = (textToken, rawHtml) => {
                if ((currentStartTag === null || currentStartTag === void 0 ? void 0 : currentStartTag.tagName) !== "style") {
                    return Promise.resolve();
                }
                const { startLine, startCol, endLine } = textToken.sourceCodeLocation;
                const numberOfLines = 1 + (endLine - startLine);
                const region = findRegion(startLine, startCol);
                const generator = new source_map_1.SourceMapGenerator();
                for (let generatedLine = 1; generatedLine <= numberOfLines; generatedLine++) {
                    generator.addMapping({
                        source: region.source,
                        generated: {
                            line: generatedLine,
                            column: 0
                        },
                        original: {
                            line: 1,
                            column: 0
                        }
                    });
                }
                generator.setSourceContent(region.source, rawHtml);
                const cssRebaser = new css_source_map_rebase_1.Rebaser({
                    map: Buffer.from(generator.toString()),
                    rebase
                });
                cssRebaser.on("rebase", (rebasedPath, resolvedPath) => {
                    eventEmitter.emit('rebase', rebasedPath, resolvedPath);
                });
                return cssRebaser.rebase(Buffer.from(rawHtml))
                    .then((result) => {
                    const { css } = result;
                    textToken.text = css.toString();
                });
            };
            const transformStartTag = (tag) => {
                const processTag = (tag) => {
                    const attributes = tag.attrs;
                    attributes.forEach((attribute) => {
                        switch (attribute.name) {
                            case 'href':
                            case 'src':
                                const url = (0, url_1.parse)(attribute.value);
                                if (isRebasable(url)) {
                                    const { startLine, startCol } = tag.sourceCodeLocation;
                                    const tagRegion = findRegion(startLine, startCol);
                                    const { source } = tagRegion;
                                    const resolvedPath = path_1.posix.join((0, path_1.dirname)(source), url.pathname);
                                    const done = (rebasedPath) => {
                                        if (rebasedPath !== false) {
                                            if (!rebasedPath) {
                                                rebasedPath = resolvedPath;
                                            }
                                            attribute.value = (0, slash_1.default)((0, path_1.join)('.', rebasedPath));
                                            eventEmitter.emit('rebase', rebasedPath, resolvedPath);
                                        }
                                    };
                                    if (!rebase) {
                                        rebase = (_url, _source, done) => {
                                            done();
                                        };
                                    }
                                    rebase(source, resolvedPath, done);
                                }
                                break;
                        }
                    });
                };
                processTag(tag);
            };
            let currentStartTag = null;
            rewritingStream.on('startTag', (startTag) => {
                defer(() => {
                    currentStartTag = startTag;
                    transformStartTag(startTag);
                    rewritingStream.emitStartTag(startTag);
                    return Promise.resolve();
                });
            });
            rewritingStream.on('text', (text, rawHtml) => {
                defer(() => {
                    return transformText(text, rawHtml)
                        .then(() => {
                        rewritingStream.emitRaw(text.text);
                    });
                });
            });
            rewritingStream.on("endTag", (endTag) => {
                defer(() => {
                    currentStartTag = null;
                    rewritingStream.emitEndTag(endTag);
                    return Promise.resolve();
                });
            });
            for (const eventName of ['doctype', 'comment']) {
                rewritingStream.on(eventName, (_token, rawHtml) => {
                    defer(() => {
                        rewritingStream.emitRaw(rawHtml);
                        return Promise.resolve();
                    });
                });
            }
            rewritingStream.write(html.toString(), () => {
                queue.then(() => {
                    rewritingStream.end();
                });
            });
        });
    };
    const rebaser = {
        rebase,
        on(event, listener) {
            eventEmitter.on(event, listener);
            return rebaser;
        }
    };
    return rebaser;
};
exports.createRebaser = createRebaser;
